package cloudduggu.com;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * Provides upload, download, view endpoints for spark & client system.
 *
 * @author  Sarvesh Kumar (CloudDuggu.com)
 * @version 1.0
 * @since   2020-08-25
 */

@RestController
public class UploadDownloadController {

	private static final Logger logger = LoggerFactory.getLogger(UploadDownloadController.class);

	static String id;
	static StringBuilder result = new StringBuilder();

//	download files from client system
	@GetMapping("/download/{fileName:.+}")
	public ResponseEntity downloadFileFromLocal(@PathVariable String fileName) {
		Resource resource = null;
		try {
			logger.info("DownloadFile: {}", fileName);
			resource = new ClassPathResource("classpath:data/"+fileName);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return ResponseEntity.ok()
				.header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + resource.getFilename() + "\"")
				.body(resource);
	}

//	upload content into client system
	@PostMapping("/upload")
	public void upload(@RequestParam String type, @RequestParam String text) {
		logger.info("type: "+type+" | text: "+text);
		if(type.equalsIgnoreCase("data")) {
			id = text;
		} else if(type.equalsIgnoreCase("result")) {
			result.delete(0, result.length());
			result.append(text);
		}
	}

//	view data content from memory object
	@GetMapping("/data")
	public String getData() {
		if(id != null) {
			String temp = id;
			id = null;
			return temp;
		}
		else
			throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Data Not Found");
	}

	//	view result content from memory object
	@GetMapping("/result")
	public String getResult() {
		String res = result.toString();
		Map<String, String> employeeMap = new HashMap<>();
		StringBuilder vertexStr = new StringBuilder();

//		create vertex string for client graph view
		Arrays.stream(res.split("\n")[0].split("\t")).forEach(
				v -> {
					String data[] = v.split(",");
					if(!vertexStr.toString().isEmpty()) vertexStr.append(",");
					vertexStr.append("{\"id\": "+data[0]+",  \"shape\": \"image\", \"image\": \"./employee/"+data[0]+".jpg\", \"label\":\""+data[1]+"\"}");
					employeeMap.put(data[0], data[1]);
				}
		);

		StringBuilder edgeStr = new StringBuilder();
		StringBuilder tableStr = new StringBuilder();

//		create edge & table string for client view page
		Arrays.stream(res.split("\n")[1].split("\t")).forEach(
				v -> {
					String data[] = v.split(",");
					if(!edgeStr.toString().isEmpty()) edgeStr.append(",");
					edgeStr.append("{\"from\": "+data[0]+", \"to\": "+data[1]+", \"label\": \""+data[2]+"\"}");
					tableStr.append("<tr><td>"+data[0]+" - "+employeeMap.get(data[0])+"</td><td>"+data[2]+"</td><td>"+data[1]+" - "+employeeMap.get(data[1])+"</td></tr>");
				}
		);

//		append all vertex, edge and table string into single result string for client page
		result = new StringBuilder();
		result.append("["+vertexStr.toString()+"]");
		result.append("\n");
		result.append("["+edgeStr+"]");
		result.append("\n");
		result.append(tableStr);

		logger.info("FinalStr: "+result.toString());

		if(result.length() > 1) {
			String temp = result.toString();
			result.delete(0, result.length());
			return temp;
		}
		else
			throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Data Not Found");
	}

}